@@ -2,7 +2,7 @@
 ## - npar: number of parameters. Wow. Really?
 ## - start(marginal, betastart): computes initial estimates
 ## - chol(tau,not.na): returns the cholesky factor of the vcov matrix (only for the not.na obs)
-## returns NULL if gamma is outside parameter space
+## returns NULL if tau is outside parameter space
 
 ## working independence correlation
 ind.cormat <- function() {
@@ -16,7 +16,7 @@ ind.cormat <- function() {
 }
 
 ## arma(p,q) correlation for time-series
-arma.cormat <- function( p , q ) {
+arma.cormat <- function( p=0 , q=0 ) {
     if(p==0 && q==0)
       return( ind.cormat() )
     start <- rep( 0 , p+q )
@@ -43,35 +43,46 @@ arma.cormat <- function( p , q ) {
 ## clustered data
 ## assume that it is not possible that all the observations inside a cluster
 ## can be missing
-cluster.cormat <- function(id, type=c("ar1", "ma1", "exch", "unstr")) { 
-    type <- match.arg(type)
-    if(!length(rle(id)$values)==length(unique(id)))
-      stop("length of 'id' must be the same of the number of observations and data must be
+cluster.cormat <- function(id, type=c("independence", "ar1", "ma1", "exchangeable", "unstructured")) {
+
+  type <- match.arg(type)
+  if(!length(rle(id)$values)==length(unique(id)))
+    stop("length of 'id' must be the same of the number of observations and data must be
 sorted in way that observations from the same cluster are contiguous")
-    ng <- 1:length(unique(id))
-    if (!(length(ng)>1)) stop("only one strata")
-    ans <- list(type=type,id=id)
-    if(type=="unstr")
-        ans$npar <- choose(max(table(id)), 2)
-    else
-        ans$npar <- 1
-    start <- rep(0, ans$npar)
-    names(start) <- paste("tau", 1:ans$npar, sep="")
-    data <- data.frame(id=id)
-    fn <- switch(type, "ar1"=function(g) nlme::corAR1(g, form= ~1|id),
-                 "ma1"=function(g) nlme::corARMA(g, form= ~1|id, p=0, q=1),
-                 "exch"=function(g) nlme::corCompSymm(g, form= ~1|id),
-                 "unstr"=function(g) nlme::corSymm(g, form= ~1|id))
-    ans$start <- function() start
-    ans$chol <- function(tau, not.na) {
-        q <- try(nlme::corMatrix(nlme::Initialize(fn(tau),data=data)),silent=TRUE)
-        if (inherits(q,"try-error")) return(NULL)
-        g <- split(not.na,id)
-        q <- try(lapply(ng,function(i) chol(q[[i]][g[[i]],g[[i]]])),silent=TRUE)
-        if (inherits(q,"try-error") ) NULL else q
-    }
-    class( ans ) <- "cormat.gcmr"
-    ans
+  ng <- 1:length(unique(id))
+  if (!(length(ng)>1)) stop("only one strata")
+  if (type=="independence") {
+    ans <- ind.cormat()
+    ans$id <- id
+    return(ans)
+  }
+  ans <- list(type=type,id=id)
+  if(type=="unstructured")
+    ans$npar <- choose(max(table(id)), 2)
+  else
+    ans$npar <- 1
+  start <- rep(0, ans$npar)
+  names(start) <- switch(type,
+                         "ar1"="ar1",
+                         "ma1"="ma1",
+                         "exchangeable"="tau",
+                         "unstructured"=paste("tau", 1:ans$npar, sep=""))
+  data <- data.frame(id=id)
+  fn <- switch(type,
+               "ar1"=function(g) nlme::corAR1(g, form= ~1|id),
+               "ma1"=function(g) nlme::corARMA(g, form= ~1|id, p=0, q=1),
+               "exchangeable"=function(g) nlme::corCompSymm(g, form= ~1|id),
+               "unstructured"=function(g) nlme::corSymm(g, form= ~1|id))
+  ans$start <- function() start
+  ans$chol <- function(tau, not.na) {
+    q <- try(nlme::corMatrix(nlme::Initialize(fn(tau),data=data)),silent=TRUE)
+    if (inherits(q,"try-error")) return(NULL)
+    g <- split(not.na,id)
+    q <- try(lapply(ng,function(i) chol(q[[i]][g[[i]],g[[i]]])),silent=TRUE)
+    if (inherits(q,"try-error") ) NULL else q
+  }
+  class( ans ) <- "cormat.gcmr"
+  ans
 }
 
 ## Matern correlation for spatial data
@@ -80,8 +91,8 @@ matern.cormat <- function(D, alpha=0.5){
   ans <- list()
   ans$npar <- 1
   start <- median(D) 
-  names(start) <- c("tau")
-  ans$start <- function( marginal, beta ) start
+  names(start) <- c("range")
+  ans$start <- function() start
   ans$chol <- function( tau, not.na ){
     if( any(tau<=0) ) return( NULL )
     S <- geoR:::matern(D, tau, alpha)
